@@ -1,14 +1,19 @@
 /*
  * -----------------------------------------------------------------
- * $Revision: 1.10 $
- * $Date: 2010/12/01 22:33:22 $
+ * $Revision$
+ * $Date$
  * -----------------------------------------------------------------
  * Programmer: Radu Serban @ LLNL
  * -----------------------------------------------------------------
- * Copyright (c) 2006, The Regents of the University of California.
- * Produced at the Lawrence Livermore National Laboratory.
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2023, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  * This is the implementation file for the optional input and output
  * functions for the adjoint module in the CVODES solver.
@@ -41,23 +46,6 @@
  * =================================================================
  */
 
-/* 
- * -----------------------------------------------------------------
- * Readibility Constants
- * -----------------------------------------------------------------
- */
-
-#define IMtype      (ca_mem->ca_IMtype)
-#define ckpntData   (ca_mem->ca_ckpntData)
-#define nbckpbs     (ca_mem->ca_nbckpbs)
-
-#define t0_         (ck_mem->ck_t0)
-#define t1_         (ck_mem->ck_t1)
-#define nst_        (ck_mem->ck_nst)
-#define q_          (ck_mem->ck_q)
-#define h_          (ck_mem->ck_h)
-#define next_       (ck_mem->ck_next)
-
 /* 
  * -----------------------------------------------------------------
  * Optional input functions for ASA
@@ -77,13 +65,13 @@ int CVodeSetAdjNoSensi(void *cvode_mem)
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeSetAdjNoSensi", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
   } 
   ca_mem = cv_mem->cv_adj_mem;
 
-  ca_mem->ca_IMstoreSensi = FALSE;
+  ca_mem->ca_IMstoreSensi = SUNFALSE;
 
   return(CV_SUCCESS);
 }
@@ -94,31 +82,33 @@ int CVodeSetAdjNoSensi(void *cvode_mem)
  * -----------------------------------------------------------------
  */
 
-int CVodeSetIterTypeB(void *cvode_mem, int which, int iterB)
+int CVodeSetNonlinearSolverB(void *cvode_mem, int which, SUNNonlinearSolver NLS)
 {
   CVodeMem cv_mem;
   CVadjMem ca_mem;
   CVodeBMem cvB_mem;
   void *cvodeB_mem;
-  int flag;
 
   /* Check if cvode_mem exists */
   if (cvode_mem == NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODEA", "CVodeSetIterTypeB", MSGCV_NO_MEM);
+    cvProcessError(NULL, CV_MEM_NULL, "CVODEA",
+                   "CVodeSetNonlinearSolverB", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
-    cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeSetIterTypeB", MSGCV_NO_ADJ);
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
+    cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA",
+                   "CVodeSetNonlinearSolverB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
-  } 
+  }
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
-  if ( which >= nbckpbs ) {
-    cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeSetIterTypeB", MSGCV_BAD_WHICH);
+  if ( which >= ca_mem->ca_nbckpbs ) {
+    cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA",
+                   "CVodeSetNonlinearSolverB", MSGCV_BAD_WHICH);
     return(CV_ILL_INPUT);
   }
 
@@ -131,9 +121,7 @@ int CVodeSetIterTypeB(void *cvode_mem, int which, int iterB)
 
   cvodeB_mem = (void *) (cvB_mem->cv_mem);
 
-  flag = CVodeSetIterType(cvodeB_mem, iterB);
-  
-  return(flag);
+  return(CVodeSetNonlinearSolver(cvodeB_mem, NLS));
 }
 
 int CVodeSetUserDataB(void *cvode_mem, int which, void *user_dataB)
@@ -150,14 +138,14 @@ int CVodeSetUserDataB(void *cvode_mem, int which, void *user_dataB)
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeSetUserDataB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
   } 
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
-  if ( which >= nbckpbs ) {
+  if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeSetUserDataB", MSGCV_BAD_WHICH);
     return(CV_ILL_INPUT);
   }
@@ -191,14 +179,14 @@ int CVodeSetMaxOrdB(void *cvode_mem, int which, int maxordB)
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeSetMaxOrdB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
   } 
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
-  if ( which >= nbckpbs ) {
+  if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeSetMaxOrdB", MSGCV_BAD_WHICH);
     return(CV_ILL_INPUT);
   }
@@ -234,14 +222,14 @@ int CVodeSetMaxNumStepsB(void *cvode_mem, int which, long int mxstepsB)
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeSetMaxNumStepsB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
   } 
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
-  if ( which >= nbckpbs ) {
+  if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeSetMaxNumStepsB", MSGCV_BAD_WHICH);
     return(CV_ILL_INPUT);
   }
@@ -276,14 +264,14 @@ int CVodeSetStabLimDetB(void *cvode_mem, int which, booleantype stldetB)
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeSetStabLimDetB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
   } 
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
-  if ( which >= nbckpbs ) {
+  if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeSetStabLimDetB", MSGCV_BAD_WHICH);
     return(CV_ILL_INPUT);
   }
@@ -318,14 +306,14 @@ int CVodeSetInitStepB(void *cvode_mem, int which, realtype hinB)
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeSetInitStepB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
   } 
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
-  if ( which >= nbckpbs ) {
+  if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeSetInitStepB", MSGCV_BAD_WHICH);
     return(CV_ILL_INPUT);
   }
@@ -360,14 +348,14 @@ int CVodeSetMinStepB(void *cvode_mem, int which, realtype hminB)
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeSetMinStepB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
   } 
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
-  if ( which >= nbckpbs ) {
+  if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeSetMinStepB", MSGCV_BAD_WHICH);
     return(CV_ILL_INPUT);
   }
@@ -402,14 +390,14 @@ int CVodeSetMaxStepB(void *cvode_mem, int which, realtype hmaxB)
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeSetMaxStepB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
   } 
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
-  if ( which >= nbckpbs ) {
+  if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeSetMaxStepB", MSGCV_BAD_WHICH);
     return(CV_ILL_INPUT);
   }
@@ -428,6 +416,46 @@ int CVodeSetMaxStepB(void *cvode_mem, int which, realtype hmaxB)
   return(flag);
 }
 
+int CVodeSetConstraintsB(void *cvode_mem, int which, N_Vector constraintsB)
+{
+  CVodeMem cv_mem;
+  CVadjMem ca_mem;
+  CVodeBMem cvB_mem;
+  void *cvodeB_mem;
+  int flag;
+
+  /* Is cvode_mem valid? */
+  if (cvode_mem == NULL) {
+    cvProcessError(NULL, CV_MEM_NULL, "CVODEA", "CVodeSetConstraintsB", MSGCV_NO_MEM);
+    return(CV_MEM_NULL);
+  }
+  cv_mem = (CVodeMem) cvode_mem;
+
+  /* Is ASA initialized? */
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
+    cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeSetConstraintsB", MSGCV_NO_ADJ);
+  }
+  ca_mem = cv_mem->cv_adj_mem;
+
+  /* Check the value of which */
+  if ( which >= ca_mem->ca_nbckpbs ) {
+    cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeSetConstraintsB", MSGCV_BAD_WHICH);
+    return(CV_ILL_INPUT);
+  }
+
+  /* Find the CVodeBMem entry in the linked list corresponding to 'which'. */
+  cvB_mem = ca_mem->cvB_mem;
+  while (cvB_mem != NULL) {
+    if ( which == cvB_mem->cv_index) break;
+    /* advance */
+    cvB_mem = cvB_mem->cv_next;
+  }
+  cvodeB_mem = (void *) cvB_mem->cv_mem;
+
+  flag = CVodeSetConstraints(cvodeB_mem, constraintsB);
+  return(flag);
+}
+
 /*
  * CVodeSetQuad*B
  *
@@ -451,14 +479,14 @@ int CVodeSetQuadErrConB(void *cvode_mem, int which, booleantype errconQB)
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeSetQuadErrConB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
   } 
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
-  if ( which >= nbckpbs ) {
+  if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeSetQuadErrConB", MSGCV_BAD_WHICH);
     return(CV_ILL_INPUT);
   }
@@ -507,14 +535,14 @@ void *CVodeGetAdjCVodeBmem(void *cvode_mem, int which)
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, 0, "CVODEA", "CVodeGetAdjCVodeBmem", MSGCV_NO_ADJ);
     return(NULL);
   } 
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
-  if ( which >= nbckpbs ) {
+  if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, 0, "CVODEA", "CVodeGetAdjCVodeBmem", MSGCV_BAD_WHICH);
     return(NULL);
   }
@@ -542,7 +570,7 @@ int CVodeGetAdjCheckPointsInfo(void *cvode_mem, CVadjCheckPointRec *ckpnt)
 {
   CVodeMem cv_mem;
   CVadjMem ca_mem;
-  CkpntMem ck_mem;
+  CVckpntMem ck_mem;
   int i;
 
   /* Check if cvode_mem exists */
@@ -553,7 +581,7 @@ int CVodeGetAdjCheckPointsInfo(void *cvode_mem, CVadjCheckPointRec *ckpnt)
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeGetAdjCheckPointsInfo", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
   } 
@@ -566,14 +594,14 @@ int CVodeGetAdjCheckPointsInfo(void *cvode_mem, CVadjCheckPointRec *ckpnt)
   while (ck_mem != NULL) {
 
     ckpnt[i].my_addr = (void *) ck_mem;
-    ckpnt[i].next_addr = (void *) next_;
-    ckpnt[i].t0 = t0_;
-    ckpnt[i].t1 = t1_;
-    ckpnt[i].nstep = nst_;
-    ckpnt[i].order = q_;
-    ckpnt[i].step = h_;
+    ckpnt[i].next_addr = (void *) ck_mem->ck_next;
+    ckpnt[i].t0 = ck_mem->ck_t0;
+    ckpnt[i].t1 = ck_mem->ck_t1;
+    ckpnt[i].nstep = ck_mem->ck_nst;
+    ckpnt[i].order = ck_mem->ck_q;
+    ckpnt[i].step = ck_mem->ck_h;
 
-    ck_mem = next_;
+    ck_mem = ck_mem->ck_next;
     i++;
 
   }
@@ -582,6 +610,14 @@ int CVodeGetAdjCheckPointsInfo(void *cvode_mem, CVadjCheckPointRec *ckpnt)
 
 }
 
+
+/* 
+ * -----------------------------------------------------------------
+ * Undocumented Development User-Callable Functions
+ * -----------------------------------------------------------------
+ */
+
+
 /*
  * CVodeGetAdjDataPointHermite
  *
@@ -594,8 +630,8 @@ int CVodeGetAdjDataPointHermite(void *cvode_mem, int which,
 {
   CVodeMem cv_mem;
   CVadjMem ca_mem;
-  DtpntMem *dt_mem;
-  HermiteDataMem content;
+  CVdtpntMem *dt_mem;
+  CVhermiteDataMem content;
 
   /* Check if cvode_mem exists */
   if (cvode_mem == NULL) {
@@ -605,7 +641,7 @@ int CVodeGetAdjDataPointHermite(void *cvode_mem, int which,
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeGetAdjDataPointHermite", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
   } 
@@ -613,14 +649,14 @@ int CVodeGetAdjDataPointHermite(void *cvode_mem, int which,
 
   dt_mem = ca_mem->dt_mem;
 
-  if (IMtype != CV_HERMITE) {
+  if (ca_mem->ca_IMtype != CV_HERMITE) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVadjGetDataPointHermite", MSGCV_WRONG_INTERP);
     return(CV_ILL_INPUT);
   }
 
   *t = dt_mem[which]->t;
 
-  content = (HermiteDataMem) (dt_mem[which]->content);
+  content = (CVhermiteDataMem) (dt_mem[which]->content);
 
   if (y != NULL)
     N_VScale(ONE, content->y, y);
@@ -643,8 +679,8 @@ int CVodeGetAdjDataPointPolynomial(void *cvode_mem, int which,
 {
   CVodeMem cv_mem;
   CVadjMem ca_mem;
-  DtpntMem *dt_mem;
-  PolynomialDataMem content;
+  CVdtpntMem *dt_mem;
+  CVpolynomialDataMem content;
 
   /* Check if cvode_mem exists */
   if (cvode_mem == NULL) {
@@ -654,7 +690,7 @@ int CVodeGetAdjDataPointPolynomial(void *cvode_mem, int which,
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeGetAdjDataPointPolynomial", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
   } 
@@ -662,14 +698,14 @@ int CVodeGetAdjDataPointPolynomial(void *cvode_mem, int which,
 
   dt_mem = ca_mem->dt_mem;
 
-  if (IMtype != CV_POLYNOMIAL) {
+  if (ca_mem->ca_IMtype != CV_POLYNOMIAL) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVadjGetDataPointPolynomial", MSGCV_WRONG_INTERP);
     return(CV_ILL_INPUT);
   }
 
   *t = dt_mem[which]->t;
 
-  content = (PolynomialDataMem) (dt_mem[which]->content);
+  content = (CVpolynomialDataMem) (dt_mem[which]->content);
 
   if (y != NULL)
     N_VScale(ONE, content->y, y);
@@ -679,11 +715,6 @@ int CVodeGetAdjDataPointPolynomial(void *cvode_mem, int which,
   return(CV_SUCCESS);
 }
 
-/* 
- * -----------------------------------------------------------------
- * UNDOCUMENTED development user-callable functions
- * -----------------------------------------------------------------
- */
 
 /*
  * CVodeGetAdjCurrentCheckPoint
@@ -704,13 +735,13 @@ int CVodeGetAdjCurrentCheckPoint(void *cvode_mem, void **addr)
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeGetAdjCurrentCheckPoint", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
   } 
   ca_mem = cv_mem->cv_adj_mem;
 
-  *addr = (void *) ckpntData;
+  *addr = (void *) ca_mem->ca_ckpntData;
 
   return(CV_SUCCESS);
 }
